#import <Cocoa/Cocoa.h>
#import "MVConnectionsController.h"
#import "MVChatWindowController.h"
#import "MVChatConnection.h"
#import "MVKeyChain.h"

static MVConnectionsController *sharedInstance = nil;

static NSString *MVToolbarConnectToggleItemIdentifier = @"MVToolbarConnectToggleItem";
static NSString *MVToolbarEditItemIdentifier = @"MVToolbarEditItem";
static NSString *MVToolbarDeleteItemIdentifier = @"MVToolbarDeleteItem";
static NSString *MVToolbarConsoleItemIdentifier = @"MVToolbarConsoleItem";
static NSString *MVToolbarJoinRoomItemIdentifier = @"MVToolbarJoinRoomItem";
static NSString *MVToolbarQueryUserItemIdentifier = @"MVToolbarQueryUserItem";

@interface MVConnectionsController (MVConnectionsControllerPrivate)
- (void) _loadBookmarkList;
- (void) _validateToolbar;
- (void) _delete:(id) sender;
@end

#pragma mark -

@implementation MVConnectionsController
+ (MVConnectionsController *) defaultManager {
	extern MVConnectionsController *sharedInstance;
	return ( sharedInstance ? sharedInstance : ( sharedInstance = [[self alloc] init] ) );
}

#pragma mark -

- (id) init {
	if( ( self = [super init] ) ) {
		_bookmarks = nil;
		_editingRooms = nil;
        _editingRow = -1;
        _editingDefaultsRow = -1;

		[NSBundle loadNibNamed:@"MVConnections" owner:self];

		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector( _refresh: ) name:MVChatConnectionWillConnectNotification object:nil];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector( _refresh: ) name:MVChatConnectionDidConnectNotification object:nil];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector( _refresh: ) name:MVChatConnectionDidNotConnectNotification object:nil];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector( _refresh: ) name:MVChatConnectionDidDisconnectNotification object:nil];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector( _refresh: ) name:MVChatConnectionNicknameAcceptedNotification object:nil];

		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector( _requestPassword: ) name:MVChatConnectionNeedPasswordNotification object:nil];

		[[NSAppleEventManager sharedAppleEventManager] setEventHandler:self andSelector:@selector( _handleURLEvent:withReplyEvent: ) forEventClass:kInternetEventClass andEventID:kAEGetURL];

		[self _loadBookmarkList];
        [self _validateToolbar];
        [[self window] makeKeyAndOrderFront:nil];
	}
	return self;
}

- (void) dealloc {
	extern MVConnectionsController *sharedInstance;

	[[self window] close];

	[_bookmarks release];
	_bookmarks = nil;

	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[[NSAppleEventManager sharedAppleEventManager] removeEventHandlerForEventClass:kInternetEventClass andEventID:kAEGetURL];

	if( self == sharedInstance ) sharedInstance = nil;
	[super dealloc];
}

- (void) awakeFromNib {
	NSToolbar *toolbar = [[[NSToolbar alloc] initWithIdentifier:@"connections.toolbar"] autorelease];
	NSTableColumn *theColumn = nil;
	id prototypeCell = nil;

	[(NSPanel *)[self window] setFloatingPanel:NO];

	theColumn = [connections tableColumnWithIdentifier:@"auto"];
	[[theColumn headerCell] setImage:[NSImage imageNamed:@"autoHeader"]];
	prototypeCell = [NSButtonCell new];
	[prototypeCell setButtonType:NSSwitchButton];
	[prototypeCell setControlSize:NSSmallControlSize];
	[theColumn setDataCell:prototypeCell];

	theColumn = [connections tableColumnWithIdentifier:@"status"];
	[[theColumn headerCell] setImage:[NSImage imageNamed:@"statusHeader"]];
	prototypeCell = [NSImageCell new];
	[prototypeCell setImageAlignment:NSImageAlignCenter];
	[theColumn setDataCell:prototypeCell];

	[toolbar setDelegate:self];
	[toolbar setAllowsUserCustomization:YES];
	[toolbar setAutosavesConfiguration:YES];
	[[self window] setToolbar:toolbar];

	[self setWindowFrameAutosaveName:@"connections"];
}

#pragma mark -

- (IBAction) showConnectionManager:(id) sender {
	[[self window] orderFront:nil];
}

#pragma mark -

/*
 * The user requested a new connection. This opens the
 * edit connection dialog with default/empty values.
 */
- (IBAction) newConnection:(id) sender {

#ifdef DEBUG_CONNECTIONS_CONTROLLER
    printf("MVConnectionsController newConnection\n");
#endif

    NSAssert(! [editConnection isVisible], @"newConnection shouldn't be called while a connection is edited");

    _editingRow = -1;
    _editingDefaultsRow = -1;
    _editingRooms = [[NSMutableArray array] retain];

    [editRemember setState:NO];
    [editAutomatic setState:NO];
    [editAddress setStringValue:@""];
    [editPort setIntValue:6667];
    [editNickname setStringValue:@""];
    [editPassword setStringValue:@""];

    [editConnection setTitle:NSLocalizedString(@"Info for new connection", "new connection info window title")];
    [editConnection center];
    [editConnection makeKeyAndOrderFront:nil];
//Markus    [openConnection center];
//Markus	[openConnection makeKeyAndOrderFront:nil];
}

#if 0 //Markus
//Markus: This method is obsolete. Put all the checks into -editConnection, better into a method checking the editConnection dialog before closing, however.
//Markus: The newConnection dialog in the nib with all it's outlets is obsolete as well.
- (IBAction) connectNewConnection:(id) sender {
	MVChatConnection *connection = nil;

    if( ! [[newNickname stringValue] length] ) {
		[[self window] makeFirstResponder:newNickname];
		NSRunCriticalAlertPanel( NSLocalizedString( @"Nickname is blank", "chat invalid nickname dialog title" ), NSLocalizedString( @"The nickname you specified is invalid because it was left blank.", "chat nickname blank dialog message" ), nil, nil, nil );
		return;
	}

	if( ! [[newAddress stringValue] length] ) {
		[[self window] makeFirstResponder:newAddress];
		NSRunCriticalAlertPanel( NSLocalizedString( @"Chat Server is blank", "chat invalid nickname dialog title" ), NSLocalizedString( @"The chat server you specified is invalid because it was left blank.", "chat server blank dialog message" ), nil, nil, nil );
		return;
	}

	if( [newPort intValue] < 0 || [newPort intValue] > 65535 ) {
		[[self window] makeFirstResponder:newPort];
		NSRunCriticalAlertPanel( NSLocalizedString( @"Chat Server Port is invalid", "chat invalid nickname dialog title" ), NSLocalizedString( @"The chat server port you specified is invalid because it can't be negative or greater than 65535.", "chat server port invalid dialog message" ), nil, nil, nil );
		return;
	}
    /* This dialog doesn't give us a password. The IRC server
       will ask us for one if needed. */
    
	{
		NSEnumerator *enumerator = [_bookmarks objectEnumerator];
		id data = nil;

		while( ( data = [enumerator nextObject] ) ) {
			if( [[(MVChatConnection *)[data objectForKey:@"connection"] server] isEqualToString:[newAddress stringValue]] &&
				[[(MVChatConnection *)[data objectForKey:@"connection"] nickname] isEqualToString:[newNickname stringValue]] ) {
				if( [(MVChatConnection *)[data objectForKey:@"connection"] isConnected] ) {
					NSRunCriticalAlertPanel( NSLocalizedString( @"Already connected", "chat invalid nickname dialog title" ), NSLocalizedString( @"The chat server with the nickname you specified is already connected to from this computer. Use another nickname if you desire multiple connections.", "chat already connected message" ), nil, nil, nil );
				} else {
					[(MVChatConnection *)[data objectForKey:@"connection"] connect];
					[openConnection orderOut:nil];
				}
				[connections selectRow:[_bookmarks indexOfObject:data] byExtendingSelection:NO];
				[[self window] makeFirstResponder:newNickname];
				return;
			}
		}
	}

	[openConnection orderOut:nil];

	connection = [[[MVChatConnection alloc] init] autorelease];
    [connection connectToServer:[newAddress stringValue] onPort:[newPort intValue] asUser:[newNickname stringValue] withPassword:@""];

	[[self window] makeKeyAndOrderFront:nil];

}
#endif

#pragma mark -

- (IBAction) messageUser:(id) sender {
	[messageUser orderOut:nil];
	[[NSApplication sharedApplication] endSheet:messageUser];

	NSAssert([connections selectedRow] != -1, @"messageUser shouldn't be called with no connection selected");

	if ([sender tag] == 1) {
		[MVChatWindowController chatWindowWithUser:[userToMessage stringValue] withConnection:[[_bookmarks objectAtIndex:[connections selectedRow]] objectForKey:@"connection"] ifExists:NO];
	}
}

- (IBAction) joinRoom:(id) sender {
    SInt16 row = [connections selectedRow];
    MVChatConnection *connection;

    NSAssert(row != -1, @"joinRoom shouldn't be called with no connection selected");

    connection = [[_bookmarks objectAtIndex:row] objectForKey:@"connection"];

	[[NSNotificationCenter defaultCenter] removeObserver:self name:MVChatConnectionGotRoomInfoNotification object:connection];

	[joinRoom orderOut:nil];
	[[NSApplication sharedApplication] endSheet:joinRoom];

	if ([sender tag] == 1) {
        [connection stopFetchingRoomList];
		[connection joinChatForRoom:[roomToJoin stringValue]];
	}
}

#pragma mark -

/*
 * The edit connection dialog is to be closed.
 * The dialog was opened with _editConnection (slightly confusing naming).
 * This will add  @"connection", @"automatic", @"rooms", @"temporary"
 * and @"savedURL" to the in-memory bookmarks as well as  @"url",
 * @"automatic" and @"rooms" to the saved bookmarks if appropriate.
 * Special care is taken to not overwrite existing connections or passwords.
 */
- (IBAction) editConnection:(id) sender {

#ifdef DEBUG_CONNECTIONS_CONTROLLER
    printf("MVConnectionsController editConnection\n");
#endif
    //Markus: check wether the user tries so add a duplicate to the bookmarks before closing the dialog. Take over all thet text field checks from -connectNewConnection as well.
    [editConnection orderOut:nil];

	if ([sender tag] == 1) {
        /* _editingRow and _editingDefaultsRow were looked up when
           opening the dialog and should be still valid. */
        NSMutableDictionary *info = nil;
		MVChatConnection *connection = nil;
        NSString *urlString;
        NSMutableArray *defaultBookmarks = [[[NSUserDefaults standardUserDefaults] objectForKey:@"MVChatBookmarks"] mutableCopy];

        if (_editingRow == -1) { // it's a new connection
            info = [NSMutableDictionary dictionary];
            [_bookmarks addObject:info];
            [connections noteNumberOfRowsChanged];
            [connections selectRow:[_bookmarks indexOfObject:info] byExtendingSelection:NO];            
            connection = [[[MVChatConnection alloc] init] autorelease];
            [info setObject:connection forKey:@"connection"];
        } else { // the connection existed before
            info = [_bookmarks objectAtIndex:_editingRow];
            connection = [info objectForKey:@"connection"];
        }

		[connection setServer:[editAddress stringValue]];
		[connection setServerPort:[editPort intValue]];
		[connection setNickname:[editNickname stringValue]];
		[connection setPassword:[editPassword stringValue]];

        [info setObject:[NSNumber numberWithBool:[editAutomatic state]] forKey:@"automatic"];
        [info setObject:[NSNumber numberWithBool:! [editRemember state]] forKey:@"temporary"];
		[info setObject:[NSArray arrayWithArray:_editingRooms] forKey:@"rooms"];

        /* Remove a eventual password of the old bookmark. */
        urlString = [info objectForKey:@"savedURL"];
        if ([urlString length] &&
            ! ([urlString isEqualToString:[[connection url] description]] ||
               [editRemember state])) {
            NSURL *url = [NSURL URLWithString:urlString];
            [[MVKeyChain defaultKeyChain] setInternetPassword:@"" forServer:[url host] account:[url user] port:[[url port] unsignedShortValue] protocol:kSecProtocolTypeIRC];
        }

        /* Eventually, transfer the bookmark to the user defaults. */
        urlString = [[connection url] description];
        if ([editRemember state]) {
            NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:urlString, @"url", [NSNumber numberWithBool:[editAutomatic state]], @"automatic", [NSArray arrayWithArray:_editingRooms], @"rooms", nil];

            if (_editingDefaultsRow != -1) {
                [defaultBookmarks replaceObjectAtIndex:_editingDefaultsRow withObject:dict];
            } else {
                [defaultBookmarks addObject: dict];
            }

            [[MVKeyChain defaultKeyChain] setInternetPassword:[connection password] forServer:[connection server] account:[connection nickname] port:[connection serverPort] protocol:kSecProtocolTypeIRC];

            [info setObject:urlString forKey:@"savedURL"];

            [[NSUserDefaults standardUserDefaults] setObject:defaultBookmarks forKey:@"MVChatBookmarks"];
            [[NSUserDefaults standardUserDefaults] synchronize];
        } else {
            [info removeObjectForKey:@"savedURL"];
            if (_editingDefaultsRow != -1) {
                [defaultBookmarks removeObjectAtIndex: _editingDefaultsRow];
            }
        }

        /* Finally, if it's a new connection, connect. */
        if (_editingRow == -1) {
            [connection connect];
        }
    }

    [connections reloadData];
	[self _validateToolbar];

    [_editingRooms release];
	_editingRooms = nil;
    _editingRow = -1;
    _editingDefaultsRow = -1;
}

- (IBAction) addRoom:(id) sender {
    
	[_editingRooms addObject:@""];
	[editRooms noteNumberOfRowsChanged];
	[editRooms selectRow:([_editingRooms count] - 1) byExtendingSelection:NO];
	[editRooms editColumn:0 row:([_editingRooms count] - 1) withEvent:nil select:NO];
}

- (IBAction) removeRoom:(id) sender {

    NSAssert([editRooms selectedRow] != -1, @"removeRoom shouldn't be called with no connection selected");
    NSAssert([editRooms editedRow] == -1, @"removeRoom shouldn't be called with a room selected");
    
	[_editingRooms removeObjectAtIndex:[editRooms selectedRow]];
	[editRooms noteNumberOfRowsChanged];
}

#pragma mark -

- (IBAction) sendPassword:(id) sender {

#ifdef DEBUG_CONNECTIONS_CONTROLLER
    printf("MVConnectionsController sendPassword\n");
#endif
    
    [nicknameAuth orderOut:nil];

	if ([sender tag] == 1) {
        [_passConnection setPassword:[authPassword stringValue]];
	}

	[_passConnection autorelease];
	_passConnection = nil;
}

#pragma mark -

- (void) handleURL:(NSURL *) url andConnectIfPossible:(BOOL) connect {

#ifdef DEBUG_CONNECTIONS_CONTROLLER
    printf("MVConnectionsController handleURL\n");
#endif

    //Markus: After reviewing the copy&paste methods, the andConnectIfPossible flag should be obsolete. it isn't handled properly anyway.
    if( [[url scheme] isEqualToString:@"irc"] ) {
		MVChatConnection *connection = nil;
		NSEnumerator *enumerator = [_bookmarks objectEnumerator];
		id data = nil;
		BOOL isRoom = NO;
		BOOL handled = NO;
		NSString *target = nil;

		/*
         * Extract a room name from the URL if possible.
         */
        if( [url fragment] ) {
			if( [[url fragment] length] > 0 ) {
				target = [url fragment];
				isRoom = YES;
			}
		} else if( [url path] && [[url path] length] >= 2 ) {
			target = [[url path] substringFromIndex:1];
			if ([target hasPrefix:@"&"] || [target hasPrefix:@"+"]) {
				isRoom = YES;
			} else {
				isRoom = NO;
			}
		}

        /*
         * Check wether a equivalent connection already exists
         * and use this if found. For URLs without port or user
         * information, use the first fitting connection available.
         *
         * Markus: since the user can temporarily change it's nick,
         *         verify against the saved URL, not against the current URL.
         */
		while( ( data = [enumerator nextObject] ) ) {
			connection = [data objectForKey:@"connection"];
			if( [[connection server] isEqualToString:[url host]] && ( ! [url user] || [[connection nickname] isEqualToString:[url user]] ) && ( ! [[url port] unsignedShortValue] || [connection serverPort] == [[url port] unsignedShortValue] ) ) {
				[connection connect];
				[connections selectRow:[_bookmarks indexOfObject:data] byExtendingSelection:NO];
                if (target) {
                    if (isRoom) {
                        [connection joinChatForRoom:target];
                    } else {
                        [MVChatWindowController chatWindowWithUser:target withConnection:connection ifExists:NO];
                    }
                } else {
                    [[self window] makeKeyAndOrderFront:nil];
                }
				handled = YES;
				break;
			}
		}

        /*
         * If there isn't already an appropriate connection, fill the
         * available data into the editConnection dialog and go on
         * similar to a new connection.
         */
        if (! handled) {
            _editingRow = -1;
            _editingDefaultsRow = -1;
            _editingRooms = [[NSMutableArray array] retain];

            [editRemember setState:NO];
            [editAutomatic setState:NO];
            if ([url host]) {
                [editAddress setStringValue:[url host]];
            } else {
                [editAddress setStringValue:@""];
            }
            if ([url port]) {
                [editPort setIntValue:[[url port] unsignedShortValue]];
            } else {
                [editPort setIntValue:6667];
            }
            if ([url user]) {
                [editNickname setStringValue:[url user]];
            } else {
                [editNickname setStringValue:@""];
            }
            if ([url password]) {
                [editPassword setStringValue:[url password]];
            } else {
                [editPassword setStringValue:@""];
            }
            if (target && isRoom) {
                [_editingRooms addObject:target];
            }
            //Markus: What about targets other than rooms, like nicks? Can we handle them here? Adding _editingNicks?

            if ([url host] && [url user]) {
                [self editConnection:editOKButton];
            } else {
                if ([url host]) {
                    [editConnection setTitle:[NSString stringWithFormat:NSLocalizedString( @"Info for: %@", "connection info window title" ), [url host]]];
                } else {
                    [editConnection setTitle:NSLocalizedString(@"Info for new connection", "new connection info window title")];
                }
                [editConnection center];
                [editConnection makeKeyAndOrderFront:nil];
            }
        }
            
	}
}

#pragma mark -

- (IBAction) cut:(id) sender {
	MVChatConnection *connection = nil;

    NSAssert([connections selectedRow] != -1, @"cut shouldn't be called with no connection selected");
    NSAssert(! [editConnection isVisible], @"cut shouldn't be called while a connection is edited");
    
	connection = [[_bookmarks objectAtIndex:[connections selectedRow]] objectForKey:@"connection"];

	[[NSPasteboard generalPasteboard] declareTypes:[NSArray arrayWithObjects:NSURLPboardType, NSStringPboardType, nil] owner:self];

	[[connection url] writeToPasteboard:[NSPasteboard generalPasteboard]];
	[[NSPasteboard generalPasteboard] setString:[[connection url] description] forType:NSStringPboardType];

	[self _delete:sender];
}

- (IBAction) copy:(id) sender {
	MVChatConnection *connection = nil;

	NSAssert([connections selectedRow] != -1, @"copy shouldn't be called with no connection selected");
    
	connection = [[_bookmarks objectAtIndex:[connections selectedRow]] objectForKey:@"connection"];
//Markus: will this really work? shouldn't we use @"savedURL"?
    
	[[NSPasteboard generalPasteboard] declareTypes:[NSArray arrayWithObjects:NSURLPboardType, NSStringPboardType, nil] owner:self];

	[[connection url] writeToPasteboard:[NSPasteboard generalPasteboard]];
	[[NSPasteboard generalPasteboard] setString:[[connection url] description] forType:NSStringPboardType];
}

- (IBAction) paste:(id) sender {
	NSURL *url = [NSURL URLFromPasteboard:[NSPasteboard generalPasteboard]];

	if( ! url ) url = [NSURL URLWithString:[[NSPasteboard generalPasteboard] stringForType:NSStringPboardType]];

	[self handleURL:url andConnectIfPossible:NO];
}

- (IBAction) clear:(id) sender {
	[self _delete:sender];
}
@end

#pragma mark -

@implementation MVConnectionsController (MVConnectionsControllerDelegate)
- (BOOL) validateMenuItem:(id <NSMenuItem>) menuItem {
	if( [menuItem action] == @selector( cut: ) ) {
		if( [connections selectedRow] == -1 || [editConnection isVisible] ) return NO;
	} else if( [menuItem action] == @selector( copy: ) ) {
		if( [connections selectedRow] == -1 ) return NO;
	} else if( [menuItem action] == @selector( clear: ) ) {
		if( [connections selectedRow] == -1 || [editConnection isVisible] ) return NO;
	}
	return YES;
}

#pragma mark -

- (int) numberOfRowsInTableView:(NSTableView *) view {

    if (view == connections) {
        return [_bookmarks count];
    } else if (view == editRooms) {
        return [_editingRooms count];
    }
    return 0;
}

- (id) tableView:(NSTableView *) view objectValueForTableColumn:(NSTableColumn *) column row:(int) row {
	if( view == connections ) {
		if( [[column identifier] isEqual:@"auto"] ) {
			return [[_bookmarks objectAtIndex:row] objectForKey:@"automatic"];
		} else if( [[column identifier] isEqual:@"address"] ) {
			return [(MVChatConnection *)[[_bookmarks objectAtIndex:row] objectForKey:@"connection"] server];
		} else if( [[column identifier] isEqual:@"port"] ) {
			return [NSNumber numberWithUnsignedShort:[(MVChatConnection *)[[_bookmarks objectAtIndex:row] objectForKey:@"connection"] serverPort]];
		} else if( [[column identifier] isEqual:@"nickname"] ) {
			return [(MVChatConnection *)[[_bookmarks objectAtIndex:row] objectForKey:@"connection"] nickname];
		}
	} else if( view == editRooms ) {
		if( [[column identifier] isEqual:@"room"] ) {
			return [_editingRooms objectAtIndex:row];
		}
	}
	return nil;
}

- (void) tableView:(NSTableView *) view willDisplayCell:(id) cell forTableColumn:(NSTableColumn *) column row:(int) row {
	if( view == connections ) {
		if( [[column identifier] isEqual:@"status"] ) {
			if( [(MVChatConnection *)[[_bookmarks objectAtIndex:row] objectForKey:@"connection"] isConnected] ) {
				if( [view editedRow] != row && ( [view selectedRow] != row || ! [[view window] isKeyWindow] || ( [view selectedRow] == row && [[view window] firstResponder] != view ) ) ) [cell setImage:[NSImage imageNamed:@"connected"]];
				else [cell setImage:[NSImage imageNamed:@"connectedSelected"]];
			} else if( [(MVChatConnection *)[[_bookmarks objectAtIndex:row] objectForKey:@"connection"] status] == MVChatConnectionConnectingStatus ) {
				if( [view editedRow] != row && ( [view selectedRow] != row || ! [[view window] isKeyWindow] || ( [view selectedRow] == row && [[view window] firstResponder] != view ) ) ) [cell setImage:[NSImage imageNamed:@"connecting"]];
				else [cell setImage:[NSImage imageNamed:@"connectingSelected"]];
			} else if( [(MVChatConnection *)[[_bookmarks objectAtIndex:row] objectForKey:@"connection"] status] == MVChatConnectionDisconnectedStatus ) {
				[cell setImage:nil];
			}
		}
	}
}

- (NSMenu *) tableView:(NSTableView *) view menuForTableColumn:(NSTableColumn *) column row:(int) row {
	if( view == connections ) {
		NSMenu *menu = [[[NSMenu alloc] initWithTitle:@""] autorelease];
		NSMenuItem *item = nil;
		BOOL connected = [(MVChatConnection *)[[_bookmarks objectAtIndex:row] objectForKey:@"connection"] isConnected];

		if( connected ) {
			item = [[[NSMenuItem alloc] initWithTitle:NSLocalizedString( @"Disconnect", "disconnect from server title" ) action:@selector( _disconnect: ) keyEquivalent:@""] autorelease];
			[item setTarget:self];
			[menu addItem:item];
		} else {
			item = [[[NSMenuItem alloc] initWithTitle:NSLocalizedString( @"Connect", "connect to server title" ) action:@selector( _connect: ) keyEquivalent:@""] autorelease];
			[item setTarget:self];
			[menu addItem:item];
		}

		[menu addItem:[NSMenuItem separatorItem]];

		item = [[[NSMenuItem alloc] initWithTitle:NSLocalizedString( @"Join Room...", "join room contextual menu item title" ) action:@selector( _joinRoom: ) keyEquivalent:@""] autorelease];
		[item setTarget:self];
		if( ! connected ) [item setAction:NULL];
		[menu addItem:item];

		item = [[[NSMenuItem alloc] initWithTitle:NSLocalizedString( @"Message User...", "message user contextual menu item title" ) action:@selector( _messageUser: ) keyEquivalent:@""] autorelease];
		[item setTarget:self];
		if( ! connected ) [item setAction:NULL];
		[menu addItem:item];

		[menu addItem:[NSMenuItem separatorItem]];

		item = [[[NSMenuItem alloc] initWithTitle:NSLocalizedString( @"Get Info", "get info contextual menu item title" ) action:@selector( _editConnection: ) keyEquivalent:@""] autorelease];
		[item setTarget:self];
		if( [editConnection isVisible] ) [item setAction:NULL];
		[menu addItem:item];

		item = [[[NSMenuItem alloc] initWithTitle:NSLocalizedString( @"Delete", "delete item title" ) action:@selector( _delete: ) keyEquivalent:@""] autorelease];
		[item setTarget:self];
		if( [editConnection isVisible] ) [item setAction:NULL];
		[menu addItem:item];

		return [[menu retain] autorelease];
	}

	return nil;
}

- (void) tableView:(NSTableView *) view setObjectValue:(id) object forTableColumn:(NSTableColumn *) column row:(int) row {
	if( view == connections ) {
		if( [[column identifier] isEqual:@"auto"] ) {
			[[_bookmarks objectAtIndex:row] setObject:object forKey:@"automatic"];
			if( [object boolValue] )
				[[_bookmarks objectAtIndex:row] setObject:[NSNumber numberWithBool:NO] forKey:@"temporary"];
		} else if( [[column identifier] isEqual:@"nickname"] ) {
			[(MVChatConnection *)[[_bookmarks objectAtIndex:row] objectForKey:@"connection"] setNickname:object];
		} else if( [[column identifier] isEqual:@"address"] ) {
			[(MVChatConnection *)[[_bookmarks objectAtIndex:row] objectForKey:@"connection"] setServer:object];
		} else if( [[column identifier] isEqual:@"port"] ) {
			[(MVChatConnection *)[[_bookmarks objectAtIndex:row] objectForKey:@"connection"] setServerPort:(unsigned short)[object intValue]];
		}
	} else if( view == editRooms ) {
		if( [[column identifier] isEqual:@"room"] ) {
			[_editingRooms replaceObjectAtIndex:row withObject:object];
		}
	}
}

- (void) tableViewSelectionDidChange:(NSNotification *) notification {
	if( [notification object] == connections ) [self _validateToolbar];
	else if( [notification object] == editRooms ) {
		[editRemoveRoom setTransparent:( [editRooms selectedRow] == -1 )];
		[editRemoveRoom highlight:NO];
	}
}

#pragma mark -

- (int) numberOfItemsInComboBox:(NSComboBox *) comboBox {
    
	if( comboBox == roomToJoin && [connections selectedRow] != -1 )
        return [[[[_bookmarks objectAtIndex:[connections selectedRow]] objectForKey:@"connection"] roomListResults] count];
	else
        return 0;
}

- (id) comboBox:(NSComboBox *) comboBox objectValueForItemAtIndex:(int) index {
	if( index == -1 ) return nil;
	if( comboBox == roomToJoin && [connections selectedRow] != -1 )
        return [[[[[_bookmarks objectAtIndex:[connections selectedRow]] objectForKey:@"connection"] roomListResults] allKeys] objectAtIndex:index];
	return nil;
}

- (unsigned int) comboBox:(NSComboBox *) comboBox indexOfItemWithStringValue:(NSString *) string {
	if( comboBox == roomToJoin && [connections selectedRow] != -1 )
        return [[[[[_bookmarks objectAtIndex:[connections selectedRow]] objectForKey:@"connection"] roomListResults] allKeys] indexOfObject:string];
	return NSNotFound;
}

- (NSString *) comboBox:(NSComboBox *) comboBox completedString:(NSString *) substring {
	if( comboBox == roomToJoin && [connections selectedRow] != -1 ) {
		NSEnumerator *enumerator = [[[[_bookmarks objectAtIndex:[connections selectedRow]] objectForKey:@"connection"] roomListResults] keyEnumerator];
		NSString *room = nil;
		while( ( room = [enumerator nextObject] ) )
			if( [room hasPrefix:substring] ) return room;
	}
	return nil;
}

#pragma mark -

- (NSToolbarItem *) toolbar:(NSToolbar *) toolbar itemForItemIdentifier:(NSString *) itemIdent willBeInsertedIntoToolbar:(BOOL) willBeInserted {
	NSToolbarItem *toolbarItem = [[[NSToolbarItem alloc] initWithItemIdentifier:itemIdent] autorelease];

	if( [itemIdent isEqualToString:MVToolbarConnectToggleItemIdentifier] ) {
		[toolbarItem setLabel:NSLocalizedString( @"Connect", "connect to server title" )];
		[toolbarItem setPaletteLabel:NSLocalizedString( @"Connect", "connect to server title" )];

		[toolbarItem setToolTip:NSLocalizedString( @"Connect to server", "connect button tooltip" )];
		[toolbarItem setImage:[NSImage imageNamed:@"connect"]];

		[toolbarItem setTarget:self];
		[toolbarItem setAction:NULL];
	} else if( [itemIdent isEqualToString:MVToolbarEditItemIdentifier] ) {
		[toolbarItem setLabel:NSLocalizedString( @"Info", "short toolbar connection info button name" )];
		[toolbarItem setPaletteLabel:NSLocalizedString( @"Connection Info", "name for connection info button in customize palette" )];

		[toolbarItem setToolTip:NSLocalizedString( @"Show connection info", "connection info button tooltip" )];
		[toolbarItem setImage:[NSImage imageNamed:@"info"]];

		[toolbarItem setTarget:self];
		[toolbarItem setAction:NULL];
	} else if( [itemIdent isEqualToString:MVToolbarDeleteItemIdentifier] ) {
		[toolbarItem setLabel:NSLocalizedString( @"Delete", "delete item title" )];
		[toolbarItem setPaletteLabel:NSLocalizedString( @"Delete Connection", "name for delete connection button in customize palette" )];

		[toolbarItem setToolTip:NSLocalizedString( @"Delete connection", "delete connection button tooltip" )];
		[toolbarItem setImage:[NSImage imageNamed:@"delete"]];

		[toolbarItem setTarget:self];
		[toolbarItem setAction:NULL];
	} else if( [itemIdent isEqualToString:MVToolbarConsoleItemIdentifier] ) {
		[toolbarItem setLabel:NSLocalizedString( @"Console", "short toolbar server console button name" )];
		[toolbarItem setPaletteLabel:NSLocalizedString( @"Server Console", "name for server console button in customize palette" )];

		[toolbarItem setToolTip:NSLocalizedString( @"Open the server console", "server console button tooltip" )];
		[toolbarItem setImage:[NSImage imageNamed:@"console"]];

		[toolbarItem setTarget:self];
		[toolbarItem setAction:NULL];
	} else if( [itemIdent isEqualToString:MVToolbarJoinRoomItemIdentifier] ) {
		[toolbarItem setLabel:NSLocalizedString( @"Join Room", "short toolbar join chat room button name" )];
		[toolbarItem setPaletteLabel:NSLocalizedString( @"Join Chat Room", "name for join chat room button in customize palette" )];

		[toolbarItem setToolTip:NSLocalizedString( @"Join a chat room", "join chat room button tooltip" )];
		[toolbarItem setImage:[NSImage imageNamed:@"joinRoom"]];

		[toolbarItem setTarget:self];
		[toolbarItem setAction:NULL];
	} else if( [itemIdent isEqualToString:MVToolbarQueryUserItemIdentifier] ) {
		[toolbarItem setLabel:NSLocalizedString( @"Message User", "toolbar message user button name" )];
		[toolbarItem setPaletteLabel:NSLocalizedString( @"Message User", "toolbar message user button name" )];

		[toolbarItem setToolTip:NSLocalizedString( @"Message a user", "message user button tooltip" )];
		[toolbarItem setImage:[NSImage imageNamed:@"messageUser"]];

		[toolbarItem setTarget:self];
		[toolbarItem setAction:NULL];
	} else toolbarItem = nil;

	return toolbarItem;
}

- (NSArray *) toolbarDefaultItemIdentifiers:(NSToolbar *) toolbar {
	return [NSArray arrayWithObjects:MVToolbarConnectToggleItemIdentifier, NSToolbarSeparatorItemIdentifier,
		MVToolbarJoinRoomItemIdentifier, MVToolbarQueryUserItemIdentifier, NSToolbarFlexibleSpaceItemIdentifier,
		MVToolbarEditItemIdentifier, MVToolbarDeleteItemIdentifier, nil];
}

- (NSArray *) toolbarAllowedItemIdentifiers:(NSToolbar *) toolbar {
	return [NSArray arrayWithObjects:NSToolbarCustomizeToolbarItemIdentifier, NSToolbarFlexibleSpaceItemIdentifier,
		NSToolbarSpaceItemIdentifier, NSToolbarSeparatorItemIdentifier, MVToolbarConnectToggleItemIdentifier,
		MVToolbarEditItemIdentifier, MVToolbarDeleteItemIdentifier, MVToolbarConsoleItemIdentifier,
		MVToolbarJoinRoomItemIdentifier, MVToolbarQueryUserItemIdentifier, nil];
}

@end

#pragma mark -

@implementation MVConnectionsController (MVConnectionsControllerPrivate)
- (void) _refresh:(NSNotification *) notification {

	[connections reloadData];
    [self _validateToolbar];
}

- (void) _refreshRooms:(NSNotification *) notification {
    
#ifdef DEBUG_CONNECTIONS_CONTROLLER
    printf("MVConnectionsController _refreshRooms\n");
#endif

    [roomToJoin noteNumberOfItemsChanged];
}

/*
 * This will load the keys @"url", @"automatic" and @"rooms" from
 * user defaults, put all of @"connection", @"automatic", @"rooms", @"temporary"
 * and @"savedURL" into the _bookmarks array and establish all connections
 * and room joins.
 * @"savedURL" is always the same as the one in user defaults and is used to
 * uniquely identify connections and to allow temporary changes to a connection
 * (like nickname changes).
 */
- (void) _loadBookmarkList {
    NSMutableArray *list = [[NSUserDefaults standardUserDefaults] objectForKey:@"MVChatBookmarks"];
    NSEnumerator *enumerator = [list objectEnumerator];
	id info = nil;

#ifdef DEBUG_CONNECTIONS_CONTROLLER
    printf("MVConnectionsController _loadBookmarkList\n");
#endif

    /*
     * We can load bookmarks only once since we whould
     * have to handle all (open) connections of the disposed set.
     */
    NSAssert(! _bookmarks, @"We can load bookmarks only once");
    _bookmarks = [[NSMutableArray alloc] init];

    while ((info = [enumerator nextObject])) {
        NSMutableDictionary *mark = [[NSMutableDictionary alloc] init];
		MVChatConnection *connection = nil;

		connection = [[[MVChatConnection alloc] initWithURL:[NSURL URLWithString:[info objectForKey:@"url"]]] autorelease];
        [connection setPassword:[[MVKeyChain defaultKeyChain] internetPasswordForServer:[connection server] account:[connection nickname] port:[connection serverPort] protocol:kSecProtocolTypeIRC]];
               
        if( [[info objectForKey:@"automatic"] boolValue] ) {
			NSEnumerator *renumerator = nil;
			id item = nil;

			[connection connect];

			renumerator = [[info objectForKey:@"rooms"] objectEnumerator];
			while( ( item = [renumerator nextObject] ) )
				[connection joinChatForRoom:item];
		}

        [mark setObject:[NSNumber numberWithBool:NO] forKey:@"temporary"];
        [mark setObject:connection forKey:@"connection"];
        [mark setObject:[info objectForKey:@"url"] forKey:@"savedURL"];
        [mark setObject:[info objectForKey:@"automatic"] forKey:@"automatic"];
        [mark setObject:[info objectForKey:@"rooms"] forKey:@"rooms"];
        [_bookmarks addObject:mark];
        [mark release];
	}

	[connections noteNumberOfRowsChanged];
    
    return;
}

- (void) _validateToolbar {
	NSEnumerator *enumerator = [[[[self window] toolbar] visibleItems] objectEnumerator];
	id item = nil;
	BOOL noneSelected = YES, connected = NO;

	if( [connections selectedRow] != -1 ) noneSelected = NO;
	if( ! noneSelected ) connected = ! ( [(MVChatConnection *)[[_bookmarks objectAtIndex:[connections selectedRow]] objectForKey:@"connection"] status] == MVChatConnectionDisconnectedStatus );
	while( ( item = [enumerator nextObject] ) ) {
		if( [[item itemIdentifier] isEqualToString:MVToolbarConnectToggleItemIdentifier] ) {
			if( noneSelected || ! connected ) {
				[item setLabel:NSLocalizedString( @"Connect", "connect to server title" )];
				[item setToolTip:NSLocalizedString( @"Connect to Server", "connect button tooltip" )];
				if( noneSelected ) [item setAction:@selector( newConnection: )];
				else [item setAction:@selector( _connect: )];
				[item setImage:[NSImage imageNamed:@"connect"]];
			} else if( connected ) {
				[item setLabel:NSLocalizedString( @"Disconnect", "disconnect from server title" )];
				[item setToolTip:NSLocalizedString( @"Disconnect from Server", "disconnect button tooltip" )];
				[item setAction:@selector( _disconnect: )];
				[item setImage:[NSImage imageNamed:@"disconnect"]];
			}
		} else if( [[item itemIdentifier] isEqualToString:MVToolbarJoinRoomItemIdentifier] ) {
			if( connected ) {
				[item setAction:@selector( _joinRoom: )];
			} else {
				[item setAction:NULL];
			}
		} else if( [[item itemIdentifier] isEqualToString:MVToolbarQueryUserItemIdentifier] ) {
			if( connected ) {
				[item setAction:@selector( _messageUser: )];
			} else {
				[item setAction:NULL];
			}
		} else if( [[item itemIdentifier] isEqualToString:MVToolbarEditItemIdentifier] ) {
			if( noneSelected || [editConnection isVisible] ) {
				[item setAction:NULL];
			} else {
				[item setAction:@selector( _editConnection: )];
			}
		} else if( [[item itemIdentifier] isEqualToString:MVToolbarDeleteItemIdentifier] ) {
			if( noneSelected || [editConnection isVisible] ) {
				[item setAction:NULL];
			} else {
				[item setAction:@selector( _delete: )];
			}
		} else if( [[item itemIdentifier] isEqualToString:MVToolbarConsoleItemIdentifier] ) {
			if( noneSelected ) {
				[item setAction:NULL];
			} else {
				[item setAction:NULL];
			}
		}
	}
}

- (void) _requestPassword:(NSNotification *) notification {
	MVChatConnection *connection = [notification object];

#ifdef DEBUG_CONNECTIONS_CONTROLLER
    printf("MVConnectionsController _requestPassword\n");
#endif

    if( [nicknameAuth isVisible] ) {
		// Do something better here, like queue requests until the current one is sent
		return;
	}

	[authAddress setObjectValue:[connection server]];
	[authNickname setObjectValue:[connection nickname]];
	[authPassword setObjectValue:@""];

	[_passConnection autorelease];
	_passConnection = [connection retain];

	[nicknameAuth center];
	[nicknameAuth orderFront:nil];
}

- (void) _connect:(id) sender {
	MVChatConnection *connection = nil;
	NSDictionary *info = nil;
	NSEnumerator *enumerator = nil;
	id item = nil;

#ifdef DEBUG_CONNECTIONS_CONTROLLER
    printf("MVConnectionsController _connect\n");
#endif

    NSAssert([connections selectedRow] != -1, @"_connect shouldn't be called with no connection selected");

	info = [_bookmarks objectAtIndex:[connections selectedRow]];
	connection = [info objectForKey:@"connection"];

	[connection connect];
    [connections reloadData];
    [self _validateToolbar];

	enumerator = [[info objectForKey:@"rooms"] objectEnumerator];
	while( ( item = [enumerator nextObject] ) )
		[connection joinChatForRoom:item];
}

- (void) _disconnect:(id) sender {
    SInt16 row = [connections selectedRow];

#ifdef DEBUG_CONNECTIONS_CONTROLLER
    printf("MVConnectionsController _disconnect\n");
#endif

    NSAssert(row != -1, @"_disconnect shouldn't be called with no connection selected");
    
	[(MVChatConnection *)[[_bookmarks objectAtIndex:row] objectForKey:@"connection"] disconnect];
    [connections reloadData];
    [self _validateToolbar];
}

- (void) _delete:(id) sender {
    NSMutableArray *defaultBookmarks = [[[NSUserDefaults standardUserDefaults] objectForKey:@"MVChatBookmarks"] mutableCopy];
    SInt16 row = [connections selectedRow];
    UInt16 i;

#ifdef DEBUG_CONNECTIONS_CONTROLLER
    printf("MVConnectionsController _delete\n");
#endif

    NSAssert (row != -1, @"_delete shouldn't be called with no connection selected");
    NSAssert (! [editConnection isVisible], @"_delete shouldn't be called while a connection is edited");

    [[[_bookmarks objectAtIndex:row] objectForKey:@"connection"] disconnect];
    
    for (i = 0; i < [defaultBookmarks count]; i++) {
        if ([[[_bookmarks objectAtIndex:row] objectForKey:@"savedURL"] isEqualToString:[[defaultBookmarks objectAtIndex:i] objectForKey:@"url"]]) {
            NSURL *url = [NSURL URLWithString:[[defaultBookmarks objectAtIndex:i] objectForKey:@"url"]];
            [[MVKeyChain defaultKeyChain] setInternetPassword:@"" forServer:[url host] account:[url user] port:[[url port] unsignedShortValue] protocol:kSecProtocolTypeIRC];
            [defaultBookmarks removeObjectAtIndex:i];
        }
    }
    [_bookmarks removeObjectAtIndex:row];

    [[NSUserDefaults standardUserDefaults] setObject:[NSArray arrayWithArray:defaultBookmarks] forKey:@"MVChatBookmarks"];
    [[NSUserDefaults standardUserDefaults] synchronize];
    
    [connections noteNumberOfRowsChanged];
    [self _validateToolbar];
}

- (void) _messageUser:(id) sender {

#ifdef DEBUG_CONNECTIONS_CONTROLLER
    printf("MVConnectionsController _messageUser\n");
#endif

    NSAssert([connections selectedRow] != -1, @"_messageUser shouldn't be called with no connection selected");
    
	[[NSApplication sharedApplication] beginSheet:messageUser modalForWindow:[self window] modalDelegate:nil didEndSelector:NULL contextInfo:NULL];
}

- (void) _joinRoom:(id) sender {
    SInt16 row = [connections selectedRow];
    
#ifdef DEBUG_CONNECTIONS_CONTROLLER
    printf("MVConnectionsController _joinRoom\n");
#endif

    NSAssert(row != -1, @"_joinRoom shouldn't be called with no connection selected");
    
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector( _refreshRooms: ) name:MVChatConnectionGotRoomInfoNotification object:[[_bookmarks objectAtIndex:row] objectForKey:@"connection"]];
	[(MVChatConnection *)[[_bookmarks objectAtIndex:row] objectForKey:@"connection"] fetchRoomList];
	[[NSApplication sharedApplication] beginSheet:joinRoom modalForWindow:[self window] modalDelegate:nil didEndSelector:NULL contextInfo:NULL];
}

/*
 * The edit connection dialog is to be opened.
 */
- (void) _editConnection:(id) sender {
    NSArray *defaultBookmarks = [[NSUserDefaults standardUserDefaults] objectForKey:@"MVChatBookmarks"];
	MVChatConnection *connection = nil;
	NSDictionary *info = nil;
    UInt16 i;

#ifdef DEBUG_CONNECTIONS_CONTROLLER
    printf("MVConnectionsController _editConnection\n");
#endif

    _editingRow = [connections selectedRow];
    NSAssert(_editingRow != -1, @"_editConnection shouldn't be called with no connection selected");
    NSAssert(! [editConnection isVisible], @"_editConnection shouldn't be called while a connection is edited");

    _editingDefaultsRow = -1;
    for (i = 0; i < [defaultBookmarks count]; i++) {
        if ([[[_bookmarks objectAtIndex:_editingRow] objectForKey:@"savedURL"] isEqualToString:[[defaultBookmarks objectAtIndex:i] objectForKey:@"url"]]) {
            _editingDefaultsRow = i;
            break;
        }
    }

	info = [_bookmarks objectAtIndex:_editingRow];
	connection = [info objectForKey:@"connection"];

    if (_editingDefaultsRow == -1) {
        [editRemember setState:NO];
    } else {
        [editRemember setState:YES];
    }
	[editAutomatic setState:[[info objectForKey:@"automatic"] boolValue]];
	[editAddress setObjectValue:[connection server]];
	[editPort setIntValue:[connection serverPort]];
	[editNickname setObjectValue:[connection nickname]];
    [editPassword setObjectValue:[[MVKeyChain defaultKeyChain] internetPasswordForServer:[connection server] account:[connection nickname] port:[connection serverPort] protocol:kSecProtocolTypeIRC]];

	_editingRooms = [[[_bookmarks objectAtIndex:_editingRow] objectForKey:@"rooms"] mutableCopy];
	[editRooms reloadData];

	[editConnection setTitle:[NSString stringWithFormat:NSLocalizedString( @"Info for: %@", "connection info window title" ), [connection server]]];

	[editConnection makeKeyAndOrderFront:nil];
	[self _validateToolbar];
}

- (void) _handleURLEvent:(NSAppleEventDescriptor *) event withReplyEvent:(NSAppleEventDescriptor *) replyEvent {
	NSURL *url = [NSURL URLWithString:[[event descriptorAtIndex:1] stringValue]];

#ifdef DEBUG_CONNECTIONS_CONTROLLER
    printf("MVConnectionsController _handleURLEvent\n");
#endif

    [self handleURL:url andConnectIfPossible:YES];
}

@end
